#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Takahiro Ikeuchi'

import os
import requests
import json
from argparse import ArgumentParser
from copy import deepcopy
from settings import get_ticket_template

class RtMattermostLogger:
    def __init__(self, auth_token, room_id, base_uri='https://chat.tecnico.ulisboa.pt/hooks/', response_format='json',
                 http_proxy='', https_proxy='', rt_base_uri=""):

        self.auth_token = auth_token
        self.room_id = room_id
        self.base_uri = base_uri
        self.response_format = response_format
        self.http_proxy = http_proxy
        self.https_proxy = https_proxy
        self.rt_base_uri = rt_base_uri

    def __post(self, post_params):
        """POST HTTP Request.

        Args:
            post_params:
            endpoint:

        Returns:
            response:

        Raises:
            TODO:

        """
        __post_params = deepcopy(post_params)

        headers = {'content-type': 'application/json'}

        proxy_dict = {}
        if self.http_proxy:
            proxy_dict['http'] = self.http_proxy
        if self.https_proxy:
            proxy_dict['https'] = self.https_proxy

        uri = self.base_uri + self.auth_token

        if not proxy_dict:
            proxy_dict = None

        http = requests.session()
        data = json.dumps(__post_params)
        response = http.post(uri, data=data, headers=headers, proxies=proxy_dict)
        http.close()

        return response

    def __send_notification(self, message, message_format='html', notify=False, color='yellow'):
        params = {
            "channel": self.room_id,
            "icon_url": "https://tickets.tecnico.ulisboa.pt/static/images/favicon.png",
            "user": "RT",
            "text": message
        }

        response = self.__post(params)

        return response.status_code

    def __get_ticket_message(self, ticket):
        return get_ticket_template(ticket)
        
    def send_ticket(self, ticket):
        message = self.__get_ticket_message(ticket)
        return self.__send_notification(message = message, message_format='html', notify=False, color='gray')

    def info(self, message, message_format='html'):

        return self.__send_notification(message=message, message_format=message_format, notify=False, color='green')

    def warn(self, message, message_format='html'):

        return self.__send_notification(message=message, message_format=message_format, notify=True, color='yellow')

    def error(self, message, message_format='html'):

        return self.__send_notification(message=message, message_format=message_format, notify=True, color='red')
