# coding=utf-8
import MySQLdb
import hashlib
import json
import logging
from sets import Set
import time
import rtkit
from rtkit.tracker import Tracker
from rtkit.authenticators import CookieAuthenticator
from rtkit.errors import RTResourceError
from settings import RT_USERNAME, RT_PASSWORD, RT_BASE_URL, CF_USERNAME, CF_PASSWORD, CF_PAGE, JIRA_USERNAME, JIRA_PASSWORD, JIRA_SERVER, CF_URL, RTJIRASTATUS_SLEEP_SECONDS, MYSQL_CONFIG
from jira.client import JIRA
import requests
from collections import defaultdict
import sys, os
import traceback

if 'host' in MYSQL_CONFIG:
	MYSQL_CONNECTION = MySQLdb.connect(host=MYSQL_CONFIG['host'], user=MYSQL_CONFIG['user'], passwd=MYSQL_CONFIG['password'], db=MYSQL_CONFIG['database'], port=MYSQL_CONFIG['port'], use_unicode=True)
else:
	MYSQL_CONNECTION = None

USER_QUERY = "select DISPLAY_NAME from USER_PROFILE where OID_USER = (SELECT OID from USER WHERE USERNAME = '%s');"

#WL_HEADERS = {
#		"X-Access-Token": WL_TOKEN,
#		"X-Client-ID": WL_CLIENTID,
#		"Content-Type": "application/json"
#	};

CF_HEADERS = {
	"Content-Type": "application/json"
}

CF_AUTH = (CF_USERNAME, CF_PASSWORD)

LOG_LEVEL = logging.DEBUG

fmt = r'[%(asctime)s] - %(levelname)s - %(message)s'
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter(fmt))
LOGGER = logging.getLogger('jirartstatus')

LOGGER.setLevel(LOG_LEVEL)

LOGGER.addHandler(handler)

CF_CONTENT_HASH = 'None'

LOGGER.debug("Init JIRA ...")
jira = JIRA(basic_auth=(JIRA_USERNAME, JIRA_PASSWORD), server=JIRA_SERVER)    # a username/password tuple
LOGGER.debug("JIRA started ...")
LOGGER.debug("Init RT Tracker ...")
tracker = Tracker(RT_BASE_URL + 'REST/1.0/', RT_USERNAME, RT_PASSWORD, CookieAuthenticator)
LOGGER.debug("RT Tracker started ...")

def get_user_displayName(username):
	if not MYSQL_CONNECTION:
		return username

	cursor = MYSQL_CONNECTION.cursor()
	cursor.execute(USER_QUERY % username)
	displayName = '-'

	for row in cursor:
		if row[0] is not None:
			displayName = row[0].encode('utf-8')
	
	cursor.close()
	return displayName

def cf_get_url(pageId):
	return CF_URL + "/rest/api/content/" + str(pageId)

def cf_get_page(pageId):
	LOGGER.debug("get CF page")
	r = requests.get(cf_get_url(pageId) + "?expand=body.storage,version,ancestors", auth=CF_AUTH)
        LOGGER.debug(r)
	return r.json()

def cf_update_page(page, content):
	global CF_CONTENT_HASH

	content_hash = hashlib.md5(content).hexdigest()

	if content_hash != CF_CONTENT_HASH:

		page['body']['storage']['value'] = content
		cur = int(page['version']['number'])
		page['version']['number'] = cur + 1

		r = requests.put(cf_get_url(page['id']), auth=CF_AUTH, data = json.dumps(page), headers = CF_HEADERS)

		CF_CONTENT_HASH = content_hash

	else:
		LOGGER.debug('nothing changed in page')

def wl_post(list_id, title):

	payload = {
		"list_id" : int(list_id),
		"title" : title
	};

	requests.post(WL_URL, headers = WL_HEADERS, data = json.dumps(payload))


def reset_list(list_id):
	
	r = requests.get("%s?list_id=%s" % (WL_URL, WL_LIST), headers = WL_HEADERS)
	tasks = r.json()
	for task in tasks:
		delete_task(task)

def delete_task(task):
	r = requests.delete("%s/%s?revision=%s" % (WL_URL, task['id'],task['revision']), headers = WL_HEADERS)
	if r.status_code != 204:
		print "error %s" % (task['id'])




def get_ticket_tr(rtTicket, issues):
	ticket = tracker.get_ticket(rtTicket)

	if ticket.status == 'stalled':
		rtTicketUrl = RT_BASE_URL + 'Ticket/Display.html?id=%s' % ticket.id
		jiraIssueUrl = JIRA_SERVER + '/browse/%s'

		jiraIssues = []

		for issue in issues:
			fixVersions = "".join([fixVersion.name for fixVersion in issue.fields.fixVersions])

			if fixVersions == "":
				jiraIssues.append("<a href='%s'>%s</a>" % (jiraIssueUrl % issue.key, issue.key))
			else:
				jiraIssues.append("<a href='%s'>%s [%s]</a>" % (jiraIssueUrl % issue.key, issue.key, fixVersions))
		return '''<tr>
					<td>%s</td>
					<td>%s</td>
					<td>%s</td>
					<td>
						<a href='%s'>%s</a>
					</td>
					<td>%s</td>
					<td>%s</td>
				  </tr>''' % (ticket.id, get_user_displayName(ticket.owner), " ".join(jiraIssues).encode('utf-8') , rtTicketUrl, ticket.subject, ticket._queue_name, ticket.status)
	return None

def get_ticket_wl(projectKey, rtTicket):
	ticket = tracker.get_ticket(rtTicket)

	if ticket.status == 'stalled':
		rtTicketUrl = RT_BASE_URL + 'Ticket/Display.html?id=%s' % ticket.id
		return "[%s] - %s : %s" % (projectKey.encode('utf-8'),rtTicketUrl,ticket.subject)

	return None


def show():

	alltickets = defaultdict(list)

	query = '''(status=Closed OR (status=Resolved AND fixVersion in (releasedVersions()))) AND "External Issue ID" is not empty order by project'''

	issues = get_jira_tickets(query)
	output = []
	if len(issues) > 0:
		for issue in issues:
			ticketId = issue.fields.customfield_10702
			if ticketId is not None:
				part = ticketId.split(",");
				if len(part) == 1:
					part = ticketId.split(" & ")
					if len(part) == 1:
						part = ticketId.split(" ")
				for p in part:
					number =''.join(i for i in p if i.isdigit())
					if len(number) > 0:
                                            alltickets[number].append(issue)
                                        else:
                                            LOGGER.error('Something wrong with issue %s and ticket number >%s< from %s' % (issue, number, ticketId))

		rows = []

		for rtTicket in alltickets.keys():
			row = get_ticket_tr(rtTicket, alltickets[rtTicket])
			if row is not None:
				rows.append(row)
				#wl_post(WL_LIST, get_ticket_wl(project.key, rtTicket))

		if len(rows) > 0:
			output.append('''<table class='table'>
						<thead>
							<tr>
								<th>RT Id</th>
								<th>RT Owner</th>
								<th>Jira Issues</th>
								<th>Subject</th>
								<th>Queue</th>
								<th>Status</th>
							</tr>
						</thead>
					<tbody>''')

			for row in rows:
				output.append(row)

			output.append("</tbody></table>")

		return output

def get_jira_tickets(query):
	return jira.search_issues(query, maxResults=1000000)

def run_it():
	page = cf_get_page(CF_PAGE)
	cf_update_page(page, "".join(show()))

def daemon():
	LOGGER.debug("start daemon ...")
	while True :
		try:
			run_it()
                        tb = "Everything ok"
		except Exception as e:
			exc_type, exc_obj, exc_tb = sys.exc_info()
			fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
			print(exc_type, fname, exc_tb.tb_lineno)
			LOGGER.error(e)
                        tb = traceback.format_exc()
		except URLError as e:
			LOGGER.error(e)
                        tb = traceback.format_exc()
                finally:
		        LOGGER.error(tb)
                LOGGER.debug("Sleep for %d seconds ..." % RTJIRASTATUS_SLEEP_SECONDS)
		time.sleep(RTJIRASTATUS_SLEEP_SECONDS)

if __name__ == '__main__':
    daemon()
    #run_it()
