#!/usr/bin/env python
# -*- coding: utf-8 -*-
#'RT/4.0.12 200 Ok\n\nid: ticket/447938

import rtkit
from rtkit.tracker import Tracker
from rtkit.authenticators import CookieAuthenticator
from rtkit.errors import RTResourceError
import time
import logging
from RtMattermostLogger import RtMattermostLogger
from settings import RT_USERNAME, RT_PASSWORD, RT_BASE_URL, CONFIGS, DAEMON_SLEEP_SECONDS
from urllib2 import URLError

LOG_LEVEL = logging.DEBUG

fmt = r'[%(asctime)s] - %(levelname)s - %(message)s'
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter(fmt))
LOGGER = logging.getLogger('rtpy')

LOGGER.setLevel(LOG_LEVEL)

LOGGER.addHandler(handler)

SAVED_DATA = {}

#print "%s %s %s" % (RT_BASE_URL, RT_USERNAME, RT_PASSWORD)
tracker = Tracker(RT_BASE_URL + 'REST/1.0/', RT_USERNAME, RT_PASSWORD, CookieAuthenticator)

def daemon():
    while True :
        for config in CONFIGS:
            try:
                update(config)
            except Exception as e:
                LOGGER.error(e)
            except URLError as e:
                LOGGER.error(e)

        LOGGER.debug("Sleep for %d seconds ..." % DAEMON_SLEEP_SECONDS)
        time.sleep(DAEMON_SLEEP_SECONDS)

def update(config):
    global SAVED_DATA
    LOGGER.debug("init update for %s" % config)
    queueId = config['queueId']
    if not queueId:
        return
    queue = tracker.get_queue(queueId)
    CURRENT_DATA = queue.search_tickets(order ="-Created")
    if queueId not in SAVED_DATA:
        SAVED_DATA[queueId] = CURRENT_DATA
        LOGGER.debug('initialize data for %(queueName)s ' % config)
        LOGGER.debug('saved data : %s', SAVED_DATA[queueId])
    else:
        check_notify(CURRENT_DATA, config)

def diff(l1, l2):
    #return sort_tickets(set(a) - set(b))
    return [x for x in l1 if x.id not in map(lambda y : y.id, l2)]

def check_notify(data, config):
    global SAVED_DATA
    queueId = config['queueId']
    if not queueId:
        return
    saved_data = SAVED_DATA[queueId]
    if len(data) != len(saved_data):
        new_tickets = diff(data, saved_data)
        for ticket in new_tickets:
            notify(config, ticket)
        SAVED_DATA[queueId] = data
    else:
        LOGGER.debug('(queue [%(queueName)s] to room [%(roomId)s]) NOP ... nothing changed' % config)

def notify(config, ticket):
    LOGGER.info("notify from queue [%s] to room [%s]] ticket [%s - %s]" % (config['queueName'], config['roomId'], ticket.id, ticket.subject))
    client = RtMattermostLogger(auth_token=config['roomToken'], room_id = config['roomId'], rt_base_uri=RT_BASE_URL)
    response = client.send_ticket(ticket)
    if response != 200:
        LOGGER.error("error contacting mattermost")

if __name__ == '__main__':
    daemon()
